import { ethers } from "ethers";
import { ChainId } from "../transfers-history/adapters/web3/model";
import { CoingeckoDataReturnType, SuggestedFeeReturnType, BridgeLimitsReturnType, AcrossBridgeStatisticsType } from "./types";
export default abstract class AbstractApiClient {
    private readonly serverlessApiUrl;
    private readonly scraperApiUrl;
    protected constructor(serverlessApiUrl: string, scraperApiUrl: string);
    protected getServerlessApiUrl(): string;
    protected getScraperApiUrl(): string;
    abstract getCoinGeckoData(l1Token: string, baseCurrency: string): Promise<CoingeckoDataReturnType>;
    abstract getSuggestedFees(amount: ethers.BigNumber, originToken: string, toChainid: ChainId, fromChainid: ChainId): Promise<SuggestedFeeReturnType>;
    abstract getBridgeLimits(token: string, fromChainId: string | ChainId, toChainId: string | ChainId): Promise<BridgeLimitsReturnType>;
    abstract getAcrossStats(): Promise<AcrossBridgeStatisticsType>;
}
