import winston from "winston";
import { SpokePoolClient } from "../SpokePoolClient";
import { HubPoolClient } from "../HubPoolClient";
import { CachedUBABundleState, ModifiedUBAFlow, SystemFeeResult, UBAClientState } from "./UBAClientTypes";
import { CachingMechanismInterface, UbaFlow, UbaInflow } from "../../interfaces";
import { BigNumber } from "ethers";
import { BaseAbstractClient } from "../BaseAbstractClient";
import UBAConfig from "../../UBAFeeCalculator/UBAFeeConfig";
import { UBAClientConfig } from "./UBAClientConfig";
export declare class UBAClientWithRefresh extends BaseAbstractClient {
    readonly clientConfig: UBAClientConfig;
    readonly tokens: string[];
    protected readonly hubPoolClient: HubPoolClient;
    readonly spokePoolClients: {
        [chainId: number]: SpokePoolClient;
    };
    protected readonly cachingClient?: CachingMechanismInterface | undefined;
    ubaClientState: UBAClientState;
    ubaBundleStates: Record<string, CachedUBABundleState>;
    enabledChainIds: number[];
    chainIdIndices: number[];
    ubaBundleBlockRanges: number[][][];
    latestBlockTimestamps: Record<number, number>;
    logger: winston.Logger;
    constructor(clientConfig: UBAClientConfig, tokens: string[], hubPoolClient: HubPoolClient, spokePoolClients: {
        [chainId: number]: SpokePoolClient;
    }, cachingClient?: CachingMechanismInterface | undefined);
    computeFeesForDeposit(deposit: UbaInflow): SystemFeeResult;
    computeBalancingFeeForNextRefund(repaymentChainId: number, refundTokenSymbol: string, amount: BigNumber): BigNumber;
    getModifiedFlows(chainId: number, tokenSymbol: string, startBlock: number, endBlock: number): ModifiedUBAFlow[];
    getKeyForBundle: (bundleBlockRanges: number[][], tokenSymbol: string, chainId: number) => string;
    getLatestBundleState(tokenSymbol: string, chainId: number): {
        flows: ModifiedUBAFlow[];
        ubaConfig: UBAConfig;
    };
    private getBundleState;
    private computeLpFee;
    private validateFlowsInBundle;
    protected validateFlow(flow: UbaFlow): Promise<ModifiedUBAFlow | undefined>;
    private getMatchingValidatedFlow;
    private getMostRecentBundleBlockRangesPerChain;
    private getUbaBundleBlockRangeContainingFlow;
    private appendValidatedFlowsToClassState;
    update(): Promise<void>;
}
